import json
import urllib3
import logging
from urllib3.util import Timeout


class UploadApi(object):
    def __init__(self, configuration=None):
        self.configuration = configuration
        host = self.configuration.host.replace('https://', '')

        self.http = urllib3.HTTPSConnectionPool(host, port=443, cert_reqs='CERT_NONE',
                                                assert_hostname=False, timeout=Timeout(connect=2.0, read=10.0))

    def upload(self, dataset_id: str, file_config: object):
        """This method will upload a file to KDP

            :param str dataset_id: ID of dataset that file will be uploaded to
            :param object file_config: JSON containing filename and path ex. { "filename": "test.csv", "path": "/path/to/file" }

        """
        response = self.http.request(
            'POST',
            self.configuration.host + '/uploads',
            headers={
                'Authorization': 'Bearer ' + self.configuration.access_token
            },
            fields={
                'datasetId': dataset_id,
                'files': (file_config['filename'], open(file_config['path'], 'rb').read()),
            }
        )

        return response



import unittest
from dataclasses import dataclass
from unittest import mock
from unittest.mock import Mock, call

import pandas as pd
from kdp_api.models import WriteBatchResponse

from kdp_connector.main import KdpConn


class TestIngest(unittest.TestCase):
    def setup(self):
        self.JWT = 'eyJhbGciOiJIUzI1NiI'
        self.workspace_id = 'abc'
        self.dataset_id = 'd851a97f-b94a-465a-ae9c-207df0a8260f'
        self.host = 'https://api.koverse.localhost'
        self.input_file = '~/documents/Test documents/WithDates.csv'
        self.write_batch_response = WriteBatchResponse(workspace=self.workspace_id,
                                                       dataset_id=self.dataset_id,
                                                       partitions=[0, 1])
        self.api_instance = Mock()
        self.api_instance.write.return_value = self.write_batch_response

    @staticmethod
    def getDataframe(size: int):
        data = []
        for x in range(size):
            data.append({'name': 'tom', 'age': 20 + x})
        return pd.DataFrame(data)

    def test_ingest_batch_sizing(self):
        @dataclass
        class TestCase:
            records: int
            batch_size: int
            expected_batches: int

        test_cases = [
            TestCase(records=1, batch_size=1, expected_batches=1),
            TestCase(records=2, batch_size=1, expected_batches=2),
            TestCase(records=10, batch_size=1, expected_batches=10),
            TestCase(records=1, batch_size=2, expected_batches=1),
            TestCase(records=1, batch_size=10, expected_batches=1),
            TestCase(records=0, batch_size=10, expected_batches=0),
            TestCase(records=10, batch_size=1000, expected_batches=1),
            TestCase(records=11, batch_size=10, expected_batches=2),
        ]
        for case in test_cases:
            self.setup()
            mock_api_instance = self.api_instance

            with(mock.patch('kdp_api.ApiClient')):
                with(mock.patch('kdp_api.Configuration')):
                    with(mock.patch('kdp_api.api.write_api.WriteApi', side_effect=[mock_api_instance])):
                        kdp_conn = KdpConn()
                        partitions_set = kdp_conn.batch_write(self.getDataframe(case.records), self.dataset_id, self.JWT,
                                                              case.batch_size)
                        self.assertEqual(len(mock_api_instance.write.mock_calls), case.expected_batches,
                                         'FAILURE: for records=' + str(case.records) +
                                         ' and batch_size=' + str(case.batch_size) +
                                         ' expected_batches did not equal ' + str(case.expected_batches))

    def test_ingest_dataframe(self):
        self.setup()
        self.batch_size = 10
        mock_api_instance = self.api_instance
        expected_write_calls = [call(dataset_id=self.dataset_id,
                                     json_record=[{'name': 'tom', 'age': 20}, {'name': 'tom', 'age': 21},
                                                  {'name': 'tom', 'age': 22}, {'name': 'tom', 'age': 23}],
                                     is_async=False)]

        with(mock.patch('kdp_api.ApiClient')):
            with(mock.patch('kdp_api.Configuration')):
                with(mock.patch('kdp_api.api.write_api.WriteApi', side_effect=[mock_api_instance])):
                    self.setup()
                    self.batch_size = 10
                    kdp_conn = KdpConn()
                    partitions_set = kdp_conn.batch_write(self.getDataframe(4), self.dataset_id, self.JWT,
                                                          self.batch_size)
                    mock_api_instance.write.assert_has_calls(expected_write_calls)

        expected_partitions_set = {0, 1}
        self.assertEqual(expected_partitions_set, partitions_set)


if __name__ == '__main__':
    unittest.main()



import sys

from setuptools import setup, find_packages

number_of_arguments = len(sys.argv)
version_parameter = sys.argv[-1]
version_provided = version_parameter.find("version=")

if version_provided > -1:
    version = version_parameter.split("version=")[1]
    sys.argv = sys.argv[0:number_of_arguments-1]
else:
    version = '0.0.1'

NAME = "kdp-python-connector"
DESCRIPTION = 'Python Connector for KDP Platform'
LONG_DESCRIPTION = 'Python Connector For Interacting with KDP Platform for various ingestion and retrieval tasks'

# Setting up
setup(
       # the name must match the folder name 'verysimplemodule'
        name=NAME,
        version=version,
        author="Koverse development team",
        author_email="developer@koverse.com",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=['pandas', 'numpy', 'kdp-api-python-client'], # add any additional packages that
        # needs to be installed along with your package. Eg: 'caer'

        keywords=['python', 'kdp'],
        # classifiers= [
        #     "Development Status :: 3 - Alpha",
        #     "Intended Audience :: Science/Research",
        #     "Intended Audience :: Developers",
        #     "Programming Language :: Python :: 3.8",
        #     "Operating System :: MacOS :: MacOS X",
        # ]
)



import kdp_api
from kdp_api.api.authentication_api import AuthenticationApi
from kdp_api.api.authentication_api import Authentication


class AuthenticationUtil(object):

    @staticmethod
    def create_authentication_token(config, email: str, password: str, workspace_id: str, strategy: str = 'local'):
        """This method will be used to create a KDP authentication token

            :param Configuration config: KDP configuration
            :param str email: User email address
            :param str password: User password
            :param str workspace_id: User workspace
            :param str strategy: Defaults to "local"

            :returns: Authentication token

            :rtype: AuthenticationDetails
        """

        with kdp_api.ApiClient(config) as api_client:
            api_instance = AuthenticationApi(api_client)

            authentication = Authentication(strategy=strategy, email=email, password=password, workspaceId=workspace_id)
            return api_instance.authentication_post(authentication=authentication)



import json
import urllib3
import logging

from urllib3.util import Timeout

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

class IndexManagementApi(object):
    def __init__(self, configuration=None):
        self.configuration = configuration
        host=self.configuration.host.replace('https://', '')

        self.http = urllib3.HTTPSConnectionPool(host, port=443, cert_reqs='CERT_NONE', assert_hostname=False,
        timeout=Timeout(connect=2.0, read=10.0))


    def modify_indexes(self, dataset_id: str, create: list, remove: list,
        autoCreateIndexes: bool = False, searchAnyField: bool = False) -> object :
        """This method will modify existing indexes on a dataset

            :param str dataset_id: ID of dataset
            :param list create: List of indexes to create
            :param list remove: List of indexes to delete
            :param bool autoCreateIndexes: Whether to automatically create indexes when data is written
            :param bool searchAnyField: Whether to automatically search any field

            :returns: Job ID

            :rtype: str
       """
        request={}
        request['action']='modifyIndexes'
        request['datasetId']=dataset_id
        request['remove']=remove
        request['create']=create
        request['autoCreateIndexes']=autoCreateIndexes
        request['searchAnyField']=searchAnyField

        encoded_data = json.dumps(request).encode('utf-8')

        logging.info('modify_indexes payload: %s' % json.dumps(request))

        result: urllib3.response.HTTPResponse = self.http.request(
            'POST',
            self.configuration.host + "/index-management",
            body=encoded_data,
            headers={
                'Content-Type': 'application/json',
                'Authorization': 'Bearer ' + self.configuration.access_token
                }
            )

        if result.status == 201:
            return json.loads(result.data)
        else:
            logging.error('unexpected response code returned. status: %s, reason: %s, message: %s' %
                (result.status, result.reason, result.msg))
            raise Exception('Failed to modify indexes')




import kdp_api
from pprint import pprint
from kdp_api.api import write_api
from kdp_api.models import WriteBatchResponse
from pandas import DataFrame

class WriteApi(object):

    def batch_write(self, config, dataset_id: str, dataframe: DataFrame, batch_size: int):
        """This method will be used to write batches of data to KDP

            :param Configuration config: Connection configuration
            :param str dataset_id: ID of the KDP dataset where the data will be written
            :param DataFrame dataframe: Data to write to KDP
            :param int batch_size: Defaults to 100

            :returns: Set of partitions data was written to

            :rtype: set
        """
        with kdp_api.ApiClient(config) as api_client:

            # Create an instance of the API class
            api_instance = write_api.WriteApi(api_client)

            partitions_set = set()

            try:
                # Convert dataframe into dict. The result is an array of json.
                json_record_array = dataframe.to_dict(orient='records')

                for i in range(0, len(json_record_array), batch_size):

                    batch = json_record_array[i:i + batch_size]

                    write_batch_response: WriteBatchResponse = api_instance.write(
                        dataset_id=dataset_id,
                        json_record=batch,
                        is_async=False
                    )

                    partitions_set.update(write_batch_response.partitions)

                return partitions_set

            except kdp_api.ApiException as e:
                pprint("Exception : %s\n" % e)
